/*
Copyright (C) 2006 Parallel Realities

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "bosses.h"

void addBoss(Properties *props)
{
	Boss *boss = entityManager->spawnBoss();
	
	boss->position = props->getVector("origin");
	
	if (props->name == "BOSS_EYEDROID_COMMANDER")
	{
		eyeDroidCommanderInit(boss);
	}
	else if (props->name == "BOSS_ION_CANNON")
	{
		ionCannonInit(boss);
	}
	else if (props->name == "BOSS_BIOTANK_1")
	{
		bioTank1Init(boss);
	}
	else if (props->name == "BOSS_BIOTANK_2")
	{
		bioTank2Init(boss);
	}
	else if (props->name == "BOSS_BIOTANK_3")
	{
		bioTank3Init(boss);
	}
	else
	{
		printf("ERROR: addBoss - '%s' not a recognised boss type\n", props->name.getText());
		exit(1);
	}
}

float bossSightTarget(Boss *boss, int range)
{
	boss->target = NULL;
	
	float distance = 0;
	float targetDistance = range;
	Vector sight;

	for (Unit *blob = (Unit*)entityManager->blobList.getFirstElement() ; blob != NULL ; blob = (Unit*)blob->next)
	{
		if (!blob->canBeDamaged())
		{
			continue;
		}
		
		distance = Math::getDistance(boss->position, blob->position);

		if (distance < targetDistance)
		{
			sight = boss->position;
			sight.z += 6;
			
			if (!unitCanSeeTarget(boss, blob, sight))
			{
				continue;
			}
			
			targetDistance = distance;
			boss->target = blob;
		}
	}
	
	return targetDistance;
}

void drawBoss(bool transparent, bool reflected)
{
	Boss *boss = (Boss*)self;
	
	if (!transparent)
	{
		glDisable(GL_BLEND);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_LIGHTING);
		
		glBindTexture(GL_TEXTURE_2D, self->definition->texture->data);
		
		glPushMatrix();
		{
			glTranslatef(boss->position.x, boss->position.y, boss->position.z);
			
			if (reflected)
			{
				glTranslatef(0, 0, -(boss->heightAboveGround * 2));
			}
			
			glRotatef(boss->rotation.x, 0.0, 0.0, 1.0);
			glRotatef(boss->rotation.y, 0.0, 1.0, 0.0);
			
			if (boss->definition->model != NULL)
			{
				glCallList(boss->definition->model->data);
			}
		}
		glPopMatrix();
	}
	else
	{
		if (boss->shieldHit)
		{
			glPushMatrix();
			{
				glTranslatef(boss->position.x, boss->position.y, boss->position.z);
			
				glDisable(GL_TEXTURE_2D);
				glEnable(GL_BLEND);
			
				GLColor c = GLColor::blue;
				c.color[3] = boss->shieldHit;
				glColor4fv(c.color);
			
				gluSphere(graphics->quadric, boss->boundingBox.getHorizontalSize(), 32, 16);
			
				glEnable(GL_TEXTURE_2D);
				glDisable(GL_BLEND);
			}
			glPopMatrix();
		
			boss->shieldHit -= (0.01 * engine->getTimeDifference(TD_LOGIC));
			Math::limit(&boss->shieldHit, 0, 2.0);
		}
	}
}

void doBosses()
{
	float percent;
	
	for (Boss *boss = (Boss*)entityManager->bossList.getFirstElement() ; boss != NULL ; boss = (Boss*)boss->next)
	{
		self = boss;
		
		bbManager->removeBox(boss);
		
		if (boss->flags & EF_DEAD)
		{
			if (!boss->dead)
			{
				fireTriggers(boss->getName(), TRIGGER_TYPE_ENTITY_KILLED);
				boss->dead = true;
			}
			
			if (!boss->referenced)
			{
				boss = (Boss*)boss->previous;
				entityManager->bossList.remove(boss->next);
			}
			
			boss->referenced = false;
			
			continue;
		}
		
		boss->referenced = false;
		
		boss->think(engine->getTimeDifference(TD_LOGIC));
		
		if ((!game->allStatic) && (!game->doingCutscene))
		{
			if (boss->performNextAction(engine->getTimeDifference(TD_LOGIC)))
			{
				if (boss->action != NULL)
				{
					boss->action();
				}
			}
		}
		
		if (withinViewableRange(boss))
		{
			entityManager->addEntityToDrawList(boss);
			
			if (boss->flags & EF_WEIGHTLESS)
			{
				addGravUnitParticles(boss, GLColor::skyBlue, GLColor::green);
			}
			
			percent = boss->getHealthPercent();
		
			if (percent <= 10)
			{
				addSmokeParticle(self->position);
			}
			else if ((percent <= 25) && ((rand() % 5) == 0))
			{
				addSmokeParticle(self->position);
			}
			else if ((percent <= 50) && ((rand() % 10) == 0))
			{
			}
			else if ((percent <= 75) && ((rand() % 15) == 0))
			{
				addSmokeParticle(self->position);
			}
		}
		
		boss->applyGravity(engine->getTimeDifference(TD_LOGIC));
		
		moveEntity(boss);
		
		bbManager->addBox(boss);
	}
}
